# -*- coding:utf-8 -*-
#---------------------------------------------------------------------
#
# Qualitative Spatial Reasoning for routing
#
# Copyright (C) 2011  Germán Carrillo (http://geotux.tuxfamily.org)
#
#---------------------------------------------------------------------
#
# licensed under the terms of GNU GPL 2
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software. If not, see <http://www.gnu.org/licenses/>.
#
#---------------------------------------------------------------------
import os

class Algorithm():
    def __init__(self, graph):
        self.graph = graph # Graph data structure with the network data
        self.sparq_dir = "/docs/downloads/sparq/" # SparQ installation dir
        self.route = {} # Dict selected dipole : Alternatives for this dipole
        self.route_order = [] # To store the ordered route (dict doesn't allow it)
        self.next_dipoles = [] # Set of next dipoles to choose from

    def run(self):
        sd="10-12" # Starting dipole 
        td="135-154" # Target dipole 
        self.next_dipoles.append(sd)
        step = 0
        while step < 20: # It's a bit awkward, I know! Just to test it step by step
            step += 1
            print "************ STEP ", step," **************"
            # Select the "next" current dipole
            if len(self.next_dipoles) > 0: 
                # Pick the last dipole and remove it as alternative dipole
                cd = self.next_dipoles[-1]
                self.route[cd] = self.next_dipoles[:-1]

            else: # No options found, go back in the route                
                next_alternatives = self.route[cd]
                del self.route[cd] # Remove the last dipole visited from the route
                self.route_order.pop() 
                if len(next_alternatives) > 0:
                    cd = next_alternatives[-1]
                    self.route[cd] = next_alternatives[:-1]
                    print "I: No options found, going back..."
                else:
                    # At this moment, it only goes back once, in a more general 
                    # solution it should be improved
                    pass                    

            self.route_order.append(cd)
            print "Current dipole:",cd

            cn = self.graph.getEdge(cd).n2.id 
            print "Current node:",cn
            r_cd_td = self.sparq_qualify(cd, td) # Relations cd-td
            #print "r_cd_td:", r_cd_td
            
            edges = self.graph.getEdges(cn) # Get candidates from the node
            for e in edges: 
                if td == e.id: # td is connected to cd, we are done!
                    self.route_order.append(td)
                    print "\n*** Eureka! The route is:", self.route_order, "***"
                    return self.route_order
                if e.id == cd: #Remove the current dipole from the list of candidates
                    edges.remove(e)

            for e in edges: 
                    print "Candidate: ", e.id

            self.next_dipoles=[]
            if len(edges) == 1:
                self.next_dipoles.append(edges[0].id)
                print "I: No need to check consistency, only one option to be followed!"
            else:
                r_c_cd = self.sparq_qualify_all(cd, edges) # Relations candidates - cd
                #print "r_c_cd:",r_c_cd
                for k,v in r_c_cd.iteritems():
                    cc = k.split("_")[1]
                    if self.sparq_consistent(v,r_cd_td,cc,td): # Check consistency
                        self.next_dipoles.append(cc)
            print "next_dipoles:", self.next_dipoles
            #print self.route, self.route_order
        return None

    def sparq_qualify(self, d1, d2):
        """ Calls the SparQ module Qualify to describe a situation """
        d1_coord = self.serialize_coord(d1)
        d2_coord = self.serialize_coord(d2)
        
        # dra-24 corresponds to Dipole Calculus
        cmd = self.sparq_dir + 'sparq qualify dra-24 all "(' + d1_coord + d2_coord + ')"'
        stream = os.popen(cmd)
        res = stream.read()
        return "(" + res.split(")")[0].split("(")[2] + ")"

    def serialize_coord(self, d):
        """ Helping function to get coordinates in SparQ syntaxis """
        e = self.graph.getEdge(d)
        x0=str(e.n1.x)
        y0=str(e.n1.y)
        x1=str(e.n2.x)
        y1=str(e.n2.y)
        
        return "(" + d + " " + x0 + " " + y0 + " " + x1 + " " + y1 + ")"

    def sparq_qualify_all(self, cd, dipoles):
        """ Calls the SparQ Qualify module for each pair candidate-cd """
        r_c_cd = {}        
        for d in dipoles:
            r_c_cd[cd+"_"+d.id] = self.sparq_qualify(cd,d.id)
        return r_c_cd

    def sparq_consistent(self, v, r_cd_td, cc, td):
        """ Checks scenario-consistency in SparQ from: 
            v: Relation current candidate - current dipole
            r_cd_td: Relation current dipole - target dipole
            cc: Current candidate
            td: Target dipole
        """
        constraint = "(ells errs lere rele)" # Our main contribution, the constraint!
        cmd = self.sparq_dir + 'sparq constraint-reasoning dra-24 scenario-consistency all "(' + v + r_cd_td + '(' + cc + ' ' + constraint + ' ' + td + '))"'
        stream = os.popen(cmd)
        res = stream.read()     
        if res != "Not consistent.\n\n":
            return True
        else:
            return False

