# -*- coding:utf-8 -*-
#---------------------------------------------------------------------
#
# Qualitative Spatial Reasoning, Routing
#
# Copyright (C) 2011 Christoph Mülligann (cmuelligann@uni-muenster.de)
# Ported to python by Germán Carrillo (http://geotux.tuxfamily.org)
#
#---------------------------------------------------------------------
#
# licensed under the terms of GNU GPL 2
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software. If not, see <http://www.gnu.org/licenses/>.
#
#---------------------------------------------------------------------

class Graph():
    """ Graph base class. It consists of nodes and edges. """
    def __init__(self):
        self.count = 0
        self.nodes = {}
        self.nodeConnections = {}
        self.edges = {}

    def addNode(self, x, y):
        id = self.count
        tempNode = Node(x,y,id)

        if self.containsNode(tempNode):
            return self.getIdNode(tempNode)
        else:
            self.nodes[id] = tempNode
            self.count += 1
            return id        

    def containsNode(self, tempNode):
        for i,j in self.nodes.iteritems():
            if tempNode.equal( j ):
                return True
        return False
   
    def getIdNode(self, tempNode):
        for i,j in self.nodes.iteritems():
            if tempNode.equal( j ):
                return i

    def getNode(self, id):
        return self.nodes[id]

    def addEdge(self, n1, n2):
        if n1 == n2:
            print "Error in Graph, n1 = n2"
            return

        if not n1 in self.nodeConnections:
            self.nodeConnections[n1] = [n2]
        else:
            self.nodeConnections[n1].append(n2)

        if not n2 in self.nodeConnections:
            self.nodeConnections[n2] = [n1]
        else:
            self.nodeConnections[n2].append(n1)
        #print "Graph:", self.nodeConnections


    def getEdges(self, n):
        lstEdges = []
        #self.edges = {}        
        n1 = self.nodes[n]
        for i in self.nodeConnections[n]:
            n2 = self.nodes[i]
            tempEdge = Edge(n1, n2)
            if not tempEdge.id in self.edges:
                self.edges[tempEdge.id] = tempEdge
            lstEdges.append( tempEdge )
        #for i in lstEdges:
            #print lstEdges[i].n1.id, lstEdges[i].n2.id, lstEdges[i].id
            #print i.n1.id, i.n2.id, i.id
        return lstEdges
    
    def getEdge(self, id):
        #nodes = id.split("-")
        #return Edge(self.nodes[int(nodes[0])], self.nodes[int(nodes[1])])
        if id in self.edges:
            return self.edges[id]
        else:
            print "Edge ", id, " is not in the edges dict..."
            nodes = id.split("-")
            tempEdge = Edge(self.nodes[int(nodes[0])], self.nodes[int(nodes[1])])
            self.edges[tempEdge.id] = tempEdge 
            return tempEdge

#class Dipole():
#    def __init__(self, edge):
#        self.startNode = edge.n1
#        self.endNode = edge.n2
#        self.id = edge.id

class Edge():
    def __init__(self, n1, n2):
        self.n1 = n1
        self.n2 = n2
        if n1.id < n2.id:
            self.id = str(n1.id) + "-" + str(n2.id) 
        elif n2.id < n1.id:
            self.id = str(n2.id) + "-" + str(n1.id) 
        else:
            print "Error in Edge, node 1 (", n1.id, ") and 2 (", n2.id, ") are the same "
            return

class Node():
    def __init__(self, x, y, id):
        self.x = x
        self.y = y
        self.id = id

    def equal(self, node):
        if self.x == node.x and self.y == node.y:
            return True
        else:
            return False

