# -*- coding:utf-8 -*-
"""
/***************************************************************************
 QualitativeRoute
                Qualitative Spatial Reasoning for routing
 Get a route based on dipole calculi
                              -------------------
        begin                : 2011-06-22
        copyright            : (C) 2011 by German Carrillo (http://geotux.tuxfamily.org)
        email                : gcarrillo@uni-muenster.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
# Import the PyQt and QGIS libraries
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
# Initialize Qt resources from file resources.py
import resources

from graph import *
from algorithm import *

class QualitativeRoute:

    def __init__(self, iface):
        # Save reference to the QGIS interface
        self.iface = iface

    def initGui(self):
        # Create action that will start plugin configuration
        self.action = QAction(QIcon(":/plugins/qualitativeroute/icon.png"), "QSKRR", self.iface.mainWindow())
        # connect the action to the run method
        QObject.connect(self.action, SIGNAL("triggered()"), self.run)

        # Add toolbar button and menu item
        self.iface.addToolBarIcon(self.action)
        self.iface.addPluginToMenu("&QSKRR", self.action)

    def unload(self):
        # Remove the plugin menu item and icon
        self.iface.removePluginMenu("&QSKRR",self.action)
        self.iface.removeToolBarIcon(self.action)

    def run(self):
        import time
        t0 = time.time()

        # Create the graph object out of the first layer in QGIS TOC
        legend = self.iface.legendInterface() 
        layer = legend.layers()[0]
        pr = layer.dataProvider()
        allAttrs = pr.attributeIndexes()
        layer.select(allAttrs)
        graph = Graph()
        for f in layer:
            line = f.geometry().asPolyline()
            n1 = graph.addNode(line[0].x()-3404000, line[0].y()-5757000) # Magic numbers to reduce the digits in calculations
            n2 = graph.addNode(line[1].x()-3404000, line[1].y()-5757000)
            graph.addEdge(n1, n2)
            print "D" + str(f.id()), n1, n2 # Dipole id, node 1, node 2

        t1 = time.time() # Graph creation

        algorithm = Algorithm(graph)
        route = algorithm.run()

        t2 = time.time() # Algorithm

        if route: # Create memory layer and add it to QGIS
            vl = QgsVectorLayer("LineString", "Calculated Route", "memory")
            pr = vl.dataProvider()
            pr.addAttributes( [ ] ) 

            # add a feature
            feat = QgsFeature()
            points = []
            for edge in route:
                n1 = graph.getNode(int(edge.split("-")[0]))
                n2 = graph.getNode(int(edge.split("-")[1]))
                p1 = QgsPoint(n1.x+3404000, n1.y+5757000) # After calculations, add again the magic numbers
                p2 = QgsPoint(n2.x+3404000, n2.y+5757000)
                points.append(p1)
                points.append(p2)
            feat.setGeometry( QgsGeometry.fromPolyline( points ) )

            pr.addFeatures( [ feat ] )
            vl.updateExtents()
            QgsMapLayerRegistry.instance().addMapLayer( vl )

            t3 = time.time() # New layer creation

            print "Graph creation:", t1 - t0, "seconds."
            print "Algorithm:", t2 - t1, "seconds."
            print "New layer creation:", t3 - t2, "seconds."
            print "TOTAL:", time.time() - t0, "seconds."

